﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.IO;


namespace PhoneBooks
{
    public class PhoneBook
    {
        private IFormatter formatter = new BinaryFormatter();
        private List<User> userslist;
        private string dataFile = @"PhoneBook.data";
        private Object objectLock = new Object();

        public void AddUser(List<User> users)
        {

            try
            {
                if (FileExists())
                {
                    var allUsers = GetUsersList();
                    foreach (var item in users)
                    {
                        allUsers.Add(item);
                    }
                    CreateUsersList(allUsers);
                }
                else
                {
                    CreateUsersList(users);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        public string RemoveUser(Guid userid)
        {
            if (!FileExists())
            {
                throw new FileNotFoundException("PhoneBook Data File Missing.");
            }

            try
            {
                int index = GetUsersList().IndexOf((from q in userslist where q.UserId == userid select q).FirstOrDefault());
                if (index != -1)
                {
                    userslist.RemoveAt(index);
                    CreateUsersList(userslist);
                    return "User deleted.";
                }
                else
                {
                    return "User not found.";
                }
            }
            catch (Exception ex)
            {

                throw ex;
            }
        }
        public string UpdateUser(Guid userid, List<User> users)
        {
            if (!FileExists())
            {
                throw new FileNotFoundException("PhoneBook Data File Missing.");
            }

            try
            {

                int index = GetUsersList().IndexOf((from q in userslist where q.UserId == userid select q).FirstOrDefault());
                if (index != -1)
                {
                    userslist[index] = users.FirstOrDefault();
                    CreateUsersList(userslist);
                    return "User updated.";

                }
                else
                {
                    return "User not found.";
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        public List<User> GetUsers(string sort = null)
        {
            try
            {
                if (!(string.IsNullOrEmpty(sort)) && (sort != "FirstName") && (sort != "LastName"))
                {
                    throw new DataColumnDoesNotExistException();
                }
                if (FileExists())
	            {
                    return (string.IsNullOrEmpty(sort) ? GetUsersList().OrderBy(o => o.UserId) : GetUsersList().OrderBy(s => s.GetType().GetProperty(sort).GetValue(s))).ToList();
	            }
                else
	            {
                    throw new FileNotFoundException("PhoneBook Data File Missing.");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        private void CreateUsersList(List<User> userslist)
        {
            lock (objectLock)
            {
                using (FileStream stream = File.Create("PhoneBook.data"))
                {
                    formatter.Serialize(stream, userslist);
                }
            }
        }
        private List<User> GetUsersList()
        {
            lock (objectLock)
            {
                using (FileStream stream = File.OpenRead("PhoneBook.data"))
                {
                    userslist = (List<User>)formatter.Deserialize(stream);
                    return userslist;
                }
            }
        }
        private bool FileExists()
        {
            lock (objectLock)
            {
                return File.Exists(dataFile) ? true : false;
            }
        }
    }

}
        public class DataColumnDoesNotExistException:Exception
        {
            public DataColumnDoesNotExistException()
                :base("Data column per which you try to sort dat does not exist.")
            {
        
            }

        }
