﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using NUnit.Framework;
using System.IO;
using PhoneBooks;

namespace PhoneBooksNunitTests
{
    [TestFixture]
    public class PhoneBooksNunitTests
    {
        [Test]
        public void aGetUsersWhereParameterIsOKButFileDoesNotExist()
        {
            File.Delete(@"PhoneBook.data");
            var phonebook = new PhoneBook();
            Assert.That(() => phonebook.GetUsers("FirstName"),
                Throws.TypeOf<FileNotFoundException>());
        }
        [Test]
        public void bInsertNewUser()
        {
            var phonebook = new PhoneBook();
            List<User> inputlist = new List<User>();
            inputlist.Add(new User
            {
                UserId = new Guid("00000000-0000-0000-0000-000000000000"),
                FirstName = "FirstNameSample1",
                LastName = "LastNameSample1",
                Phones = new List<Phone> { 
                                    new Phone{Cellphone = "11", Home = "111", Work = "111"},
                                    new Phone{Cellphone = "11", Home = "111", Work = "111"},
                                    new Phone{Cellphone = "11", Home = "111", Work = "111"},
                                            },
            });
            List<User> outputlist = new List<User>();
            outputlist = inputlist;
            phonebook.AddUser(inputlist);
            Assert.AreEqual(inputlist, outputlist);
        }
        [Test]
        public void cUpdateUserWithPredefinedIDWhichWillReturnUpdated()
        {
            List<User> inputlist = new List<User>();
            inputlist.Add(new User
            {
                UserId = new Guid("00000000-0000-0000-0000-000000000000"),
                FirstName = "FirstNameSample111",
                LastName = "LastNameSample111",
                Phones = new List<Phone> { 
                                    new Phone{Cellphone = "1111", Home = "11111", Work = "11111"},
                                    new Phone{Cellphone = "1111", Home = "11111", Work = "11111"},
                                    new Phone{Cellphone = "1111", Home = "11111", Work = "11111"},
                                            },
            });
            List<User> outputlist = new List<User>();

            var phonebook = new PhoneBook();
            Assert.AreEqual("User updated.", phonebook.UpdateUser(new Guid("00000000-0000-0000-0000-000000000000"), inputlist));
        }

        [Test]
        public void dGetUsersWhereParameterIsSortinParammeter()
        {
            var phonebook = new PhoneBook();
            Assert.IsInstanceOf(typeof(List<User>), phonebook.GetUsers(""));
        }

        [Test]
        public void eGetUsersWhereParameterIsSortInParammeterWhichDoesNotExistInClass()
        {
            var phonebook = new PhoneBook();
            Assert.That(() => phonebook.GetUsers("WrongColumnName"),
                Throws.TypeOf<DataColumnDoesNotExistException>());
        }
        [Test]
        public void fRemoveUserWithGenericIDwhichWillReturnNotFound()
        {
            var phonebook = new PhoneBook();
            Assert.AreEqual("User not found.", phonebook.RemoveUser(Guid.NewGuid()));
        }

        [Test]
        public void gUpdateUserWithWithGenericIDwhichWillReturnNotFound()
        {
            List<User> inputlist = new List<User>();
            inputlist.Add(new User
            {
                FirstName = "FirstNameSample111",
                LastName = "LastNameSample111",
                Phones = new List<Phone> { 
                                    new Phone{Cellphone = "1111", Home = "11111", Work = "11111"},
                                    new Phone{Cellphone = "1111", Home = "11111", Work = "11111"},
                                    new Phone{Cellphone = "1111", Home = "11111", Work = "11111"},
                                            },
            });
            List<User> outputlist = new List<User>();
            var phonebook = new PhoneBook();
            Assert.AreEqual("User not found.", phonebook.UpdateUser(Guid.NewGuid(), inputlist));
        }
        [Test]
        public void hRemoveUserWithPredefinedIDWhichWillReturnDeleted()
        {
            var phonebook = new PhoneBook();
            Assert.AreEqual("User deleted.", phonebook.RemoveUser(new Guid("00000000-0000-0000-0000-000000000000")));
        }



    }
}
